/*!
 * \file 
 * \brief Rs232Spi Click example
 * 
 * # Description
 * This example showcases how to initialize and use the RS232 SPI click. The click has a uni-
 * versal asynchronous transceiver which uses a SPI serial interface to communicate with the 
 * MCU. In order for this example to work, 2 clicks are needed - a receiver and a transmitter.
 *
 * The demo application is composed of two sections :
 * 
 * ## Application Init 
 * This function initializes and configures the logger and click modules. Additional configura-
 * ting is done in the default_cfg(...) function.
 * 
 * ## Application Task  
 * This function receives and displays UART data in the "read mode" and sends the predefined 
 * message in the "write mode".
 * 
 * \author MikroE Team
 *
 */
// ------------------------------------------------------------------- INCLUDES

#include "board.h"
#include "log.h"
#include "rs232spi.h"

// ------------------------------------------------------------------ VARIABLES

static rs232spi_t rs232spi;
static log_t logger;

static const uint8_t message[ 9 ] = { 'M', 'i', 'k', 'r', 'o', 'E', 13, 10, 0 };

// ------------------------------------------------------ APPLICATION FUNCTIONS

void application_init ( )
{
    log_cfg_t log_cfg;
    rs232spi_cfg_t cfg;

    //  Logger initialization.

    log_cfg.level = LOG_LEVEL_DEBUG;
    LOG_MAP_USB_UART( log_cfg );
    log_init( &logger, &log_cfg );
    log_info( &logger, "---- Application Init ----" );

    //  Click initialization.

    rs232spi_cfg_setup( &cfg );
    RS232SPI_MAP_MIKROBUS( cfg, MIKROBUS_1 );
    rs232spi_init( &rs232spi, &cfg );
    Delay_100ms( );
    rs232spi_digital_write_rst( &rs232spi, 1 );
    Delay_10ms( );
    rs232spi_default_cfg( &rs232spi, 19200 );
    Delay_10ms( );
    rs232spi_flush( &rs232spi );
    Delay_100ms( );
}

void application_task ( )
{
    uint8_t mode = 0;
    uint8_t cnt;

    if ( mode == 1 )
    {
        if ( rs232spi_data_ready( &rs232spi ) != 0 )
        {
            log_printf( &logger, "%c\r\n", rs232spi_transfer( &rs232spi, RS232SPI_CMD_READ_DATA ) );
            Delay_100ms( );
        }
    }
    else if ( mode == 0 )
    {
        for ( cnt = 0; cnt < 9; cnt++ )
        {
            rs232spi_data_write( &rs232spi, message[ cnt ] );
            if ( cnt < 6 )
            {
                Delay_100ms( );
            }
        }

        Delay_ms( 2000 );
    }
}

void main ( )
{
    application_init( );

    for ( ; ; )
    {
        application_task( );
    }
}

// ------------------------------------------------------------------------ END
